/*      
        alph: Piper program implementing and analysing traditional and
	historic c(ai)phers designed to be used in conjuction with other
	programs.

        Copyright (C) 2004  Corcalciuc V. Horia

        This library is free software; you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public
        License as published by the Free Software Foundation; either
        version 2.1 of the License, or (at your option) any later version.

        This library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
        Lesser General Public License for more details.

        You should have received a copy of the GNU Lesser General Public
        License along with this library; if not, write to the Free Software
        Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

        Dedicated to Rudolf.
*/


/* System. */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <time.h>
#include <string.h>
#include <locale.h>
#include <sys/time.h>
#include <math.h>
#include <popt.h>

/* Our own. */
#include "alphcy.h"
#include "alphan.h"
#include "anstats.h"
#include "config.h"

int main(const int argc, const char **argv) {

	poptContext optCon;
	char *optBuf;
	int result;
	int itra=0;
	int status, pid;
	char *key;
	struct timeval *start, *stop;
	char *indata;
	FILE *fp;
	
	typedef enum {
	 false=0,
	 true
  } flip;
  flip check=false;
  
  alphize an;
      
	const struct poptOption optionT[] = {
		{ "\b\b  \nCyphers:", 0, 0, NULL, 0, NULL},
		{ "caesar",	0, 0, NULL, 'c', "CAESAR", NULL},
		{ "atbash", 0, 0, NULL, 'b', "ATBASH", NULL},
		{ "rot13", 0, 0, NULL, '3', "ROT-13", NULL},
		{ "vigenere", 0, POPT_ARG_STRING, &key, 'v', "VIGENERE", "KEY"},
		{ "vernam", 0, POPT_ARG_STRING, &key, 'k', "VERNAM", "FILE"},
		{ "playfair", 0, POPT_ARG_STRING, &key, 'f', "PLAYFAIR", "KEY"},
		{ "\b\b  \nOptional:", 0, 0, NULL, 0, NULL},
		{ NULL, 'c', 0, NULL, 'a', "efficiency checks", NULL},
		{ "\b\b  \nMandatory:", 0, 0, NULL, 0, NULL},
		{ NULL, 'd', 0, NULL, 'd', "decrypt text", NULL},
		{ NULL, 'x', 0, NULL, 'x', "encrypt text", NULL},
		{ "\b\b  \nHelp:", 0, 0, NULL, 0, NULL },
		{ "help", 'h', 0, NULL, 'h', "show help message", NULL},
		{ "man", 0, 0, NULL, 'm', "display manual page", NULL},
		{NULL, 0, 0, NULL, 0}
	};

	optBuf = calloc(argc, sizeof(char));
	indata = calloc(BUFSIZ, sizeof(char));
	key = calloc(BUFSIZ, sizeof(char));

	setvbuf(stdout, (char *) NULL, _IONBF, 0);
	setvbuf(stdin, (char *) NULL, _IONBF, 0);
	setlocale(LC_ALL, "");

	optCon = poptGetContext(NULL, argc, argv, optionT, 0);

		while((result = poptGetNextOpt(optCon)) >=0)
		{
			switch(result)
			{
				case 'h':
				optBuf[itra++]='h';
				break;
				case 'm':
				optBuf[itra++]='m';
				break;
				case 'c':
				optBuf[itra++]='c';
				break;
				case 'b':
				optBuf[itra++]='b';
				break;
				case '3':
				optBuf[itra++]='3';
				break;
				case 'v':
				optBuf[itra++]='v';
				break;
				case 'k':
				optBuf[itra++]='k';
				break;
				case 'f':
				optBuf[itra++]='f';
				break;
				case 'a':
				optBuf[itra++]='a';
				break;
				case 'p':
				optBuf[itra++]='p';
				break;
				case 'd':
				optBuf[itra++]='d';
				break;
				case 'x':
				optBuf[itra++]='x';
				break;
			}
		}

		if (result < -1)
		{
			fprintf (stderr, "%s: %s\n", poptBadOption (optCon, POPT_BADOPTION_NOALIAS),
poptStrerror (result));
			exit(1);
		}
		else if (argc <2) {
			fprintf (stderr, "%s: You must select an action.\n", PACKAGE_NAME);
			exit(1);
		}

		if ( strchr(optBuf, 'h') != NULL ) {
			poptPrintHelp(optCon, stderr, 0);
			exit(0);
		}

		if (strchr(optBuf, 'm') != NULL) {
			if((pid=vfork())==0)
				execlp("man", "-a", PACKAGE_NAME, NULL);
			else if(pid<0) fprintf(stdout, "%s", strerror(errno));
			while(wait(&status)<0);
			exit(0);
		}

		for(itra=0; itra<=BUFSIZ; itra++)
			if(!feof(stdin))
				indata[itra]=fgetc(stdin);
			else {
				indata[itra-1]='\0';
				break;
			}

	if ( strchr(optBuf, 'd') != NULL )
		{
			al_dec=true;
		}
	else if (strchr(optBuf, 'x') != NULL )
		{
			al_dec=false;
		}
	else if (strchr(optBuf, 'a') != NULL )
		{
			check=true;
		}
	else
			exit(1);
  
	if ( strchr (optBuf, 'c') != NULL )
		{
		  if(check)
        caip = &caesar;
			else
			 caesar ( indata );
		}
	if ( strchr(optBuf, 'b') != NULL )
		{
		  if(check)
  		  caip = &atbash;
  		else
        atbash ( indata );
		}
	if (strchr(optBuf, '3') != NULL )
	  {
	    if(check)
	      caip = &rot13;
	    else
	      rot13 ( indata );
	  }
	if ( strchr(optBuf, 'v') != NULL )
		{
		  if(check)
			  caip = &vigenere;
			else
        vigenere ( indata, key );
		}
	if ( strchr(optBuf, 'k') != NULL )
		{
		  if((fp=fopen(key, "r"))==NULL)
         fprintf(stdout, "%s : %s", PACKAGE_NAME, KEYB);
           return 1;
      if(check)
  		  caip = &vernam;
  		else
        vernam ( indata, key );
		}
	if ( strchr(optBuf, 'f') != NULL )
		{
		  if(check)
			 caip = &playfair;
			else
			 playfair ( indata, key );
		}

	if(!check)
		fprintf(stdout, "%s", indata);
	else
	 an=cystats(indata, key);
	 
	if(check)
	 {
	   fprintf(stdout, "---\t\tSpeed\t\t---\n");
	   fprintf(stdout, "   Plain data size: %db\n", an.p_size);
	   fprintf(stdout, "   Crypt data size: %db\n", an.c_size);
	   fprintf(stdout, "   Plain data length: %d\n", an.p_len);
	   fprintf(stdout, "   Crypt data length: %d\n", an.c_len);
	   fprintf(stdout, "---\t\tTime\t\t---\n");
	   fprintf(stdout, "   Encrypt time: %d%cs\n", an.t_encrypt, 230);
	   fprintf(stdout, "   Decrypt time: %d%cs\n", an.t_decrypt, 230);
	   fprintf(stdout, "---\t\tCrypto\t\t---\n");
	   fprintf(stdout, "   Comparison blocks: \n\n   ");
	   for(itra=0; itra<26; itra++)
  	   fprintf(stdout, "%c", itra+'a');
	   fprintf(stdout, " alpha\n   ");
	   for(itra=0; itra<26; itra++)
	     fprintf(stdout, "%d", an.a26[itra]);
	   fprintf(stdout, " found\n   ");
	   for(itra=0; itra<26; itra++)
	     fprintf(stdout, "%c", english[itra]);
	   fprintf(stdout, " stats\n\n");
	   fprintf(stdout, "   Avgerage Speed: %0.1f%cs\n", an.speed, 230);
	   fprintf(stdout, "   Real consistency: %d%%\n", an.consistency);
	   fprintf(stdout, "   Relative sensitivity: %d\n", an.sensitivity);
	   fprintf(stdout, "   Armor: %0.3f\n", an.armor);
	 }
	 
	poptFreeContext(optCon);
	exit(0);
}
